import { PostToConnectionCommand } from '@aws-sdk/client-apigatewaymanagementapi';
import { callTRPCProcedure } from '@trpc/server';
import { getApiGatewayManagementApiClient } from '../getApiGatewayManagementApiClient';
import { getConnection } from '../getConnection';
export const awsLambdaTrpcSubscriptionRequestHandler = (opts) => {
    const { router } = opts;
    return async (event) => {
        const connection = getConnection(event);
        if (!event.body || event.body === '[]')
            return { statusCode: 200, body: '' };
        const request = JSON.parse(event.body);
        const sendMessage = async (response) => {
            await getApiGatewayManagementApiClient(connection.endpoint).send(new PostToConnectionCommand({
                ConnectionId: connection.connectionId,
                Data: JSON.stringify(response),
            }));
        };
        const stopConnection = async () => {
            await sendMessage({
                id: request.id,
                result: { type: 'stopped' },
            });
        };
        try {
            if (request.method === 'subscription.stop') {
                await stopConnection();
            }
            if (request.method === 'subscription') {
                await sendMessage({
                    id: request.id,
                    result: { type: 'started' },
                });
                const sub = (await callTRPCProcedure({
                    procedures: router._def.procedures,
                    path: request.params.path,
                    type: 'subscription',
                    ctx: {},
                    getRawInput: async () => request.params.input,
                }));
                const messageQueue = [];
                sub.subscribe({
                    next: (data) => {
                        messageQueue.push({
                            id: request.id,
                            result: {
                                type: 'data',
                                data: data,
                            },
                        });
                    },
                    error: (err) => {
                        console.error(err);
                        messageQueue.push({
                            id: request.id,
                            result: { type: 'stopped' },
                        });
                    },
                    complete: () => {
                        messageQueue.push({
                            id: request.id,
                            result: { type: 'stopped' },
                        });
                    },
                });
                let stopped = false;
                while (!stopped) {
                    const message = messageQueue.shift();
                    if (!message)
                        continue;
                    await sendMessage(message);
                    stopped = message.result.type === 'stopped';
                }
            }
        }
        catch (err) {
            console.error(err);
            await stopConnection();
        }
        return {
            statusCode: 200,
            body: '',
        };
    };
};
//# sourceMappingURL=awsLambdaTrpcSubscriptionRequestHandler.js.map